/*
	file automatically generated by make_test_files.pl
	Fri Jun 14 15:21:41 2002
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DISTR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_DISTR
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_distr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

static int COMPARE_SAMPLE_SIZE = 500;
static int VIOLATE_SAMPLE_SIZE = 20;

double pdf(double x, UNUR_DISTR *distr);
double dpdf(double x, UNUR_DISTR *distr);
double cdf(double x, UNUR_DISTR *distr);

double pmf(int k, UNUR_DISTR *distr);
double icdf(int k, UNUR_DISTR *distr);

UNUR_DISTR *get_distr_with_pv( void );






/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_distr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_distr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(7823)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("distr: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 45, (unur_distr_set_name( distr, "nix" )) );
n_tests_failed += check_errorcode( TESTLOG, 45, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 48, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 48, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 51, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 54, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 54, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 57, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 57, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 60, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 60, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 63, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 63, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 66, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 66, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 69, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 69, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 72, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 72, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 75, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 75, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 78, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 78, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 81, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 81, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 84, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 84, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 87, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 87, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
	distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 94, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 94, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 97, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 97, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 100, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 100, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 103, (unur_distr_cont_set_pdfstr( distr, "exp(-x)" )) );
n_tests_failed += check_errorcode( TESTLOG, 103, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 106, (unur_distr_cont_set_cdfstr( distr, "exp(-x)" )) );
n_tests_failed += check_errorcode( TESTLOG, 106, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 109, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 109, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 112, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 112, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 115, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 115, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 118, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 118, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 125, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 125, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 128, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 128, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 131, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 131, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 134, (unur_distr_discr_set_pmfstr( distr, "exp(-k)" )) );
n_tests_failed += check_errorcode( TESTLOG, 134, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 137, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 137, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 140, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 140, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 143, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 143, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 146, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 146, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 153, (unur_distr_cont_set_pdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 153, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 156, (unur_distr_cont_set_dpdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 156, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 159, (unur_distr_cont_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 159, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 162, (unur_distr_cont_set_pdfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 162, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 165, (unur_distr_cont_set_pdfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 165, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 168, (unur_distr_cont_set_cdfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 168, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 175, (unur_distr_discr_set_pmf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 175, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 178, (unur_distr_discr_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 178, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 181, (unur_distr_discr_set_pmfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 181, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 184, (unur_distr_discr_set_pmfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 184, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 192, (unur_distr_cont_set_pdfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 192, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 195, (unur_distr_cont_set_pdfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 198, (unur_distr_cont_set_domain( distr, 0., -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 198, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 201, (unur_distr_cont_set_pdfarea( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 201, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 204, (unur_distr_cont_set_pdfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 204, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 207, (unur_distr_cont_set_pdfstr( distr, "x * y * z" )) );
n_tests_failed += check_errorcode( TESTLOG, 207, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 210, (unur_distr_cont_set_pdfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 210, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 213, (unur_distr_cont_set_cdfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 213, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 216, (unur_distr_cont_set_cdfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 216, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 224, (unur_distr_discr_set_pmfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 224, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 227, (unur_distr_discr_set_pmfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 227, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 230, (unur_distr_discr_set_domain( distr, 10, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 230, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 233, (unur_distr_discr_set_pmfsum( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 233, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 236, (unur_distr_discr_set_pmfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 236, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 239, (unur_distr_discr_set_pmfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 239, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   double a, b;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 251, ((void*) unur_distr_get_name( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 251, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 254, (unur_distr_get_type( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 254, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 257, (unur_distr_is_cont( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 257, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 260, (unur_distr_is_discr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 260, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 263, ((void*) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 263, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 266, ((void*) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 266, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 269, ((void*) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 269, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 272, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 272, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 275, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 275, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 278, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 278, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 281, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 281, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 284, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 284, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 287, ((void*) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 287, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 290, ((void*) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 290, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 293, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 293, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 296, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 296, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   double a,b;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 304, ((void*) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 304, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 307, ((void*) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 307, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 310, ((void*) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 310, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 313, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 313, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 316, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 316, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 319, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 319, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 322, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 322, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   int ia, ib;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 329, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 329, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 332, (unur_distr_discr_get_domain( distr, &ia, &ib )) );
n_tests_failed += check_errorcode( TESTLOG, 332, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 340, ((void*) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 340, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 343, ((void*) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 343, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 346, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 346, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 349, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 349, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* parameters not unknwon */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 355, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 355, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 358, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 358, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}

/* parameters not unknwon */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 364, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 364, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 375, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 375, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 378, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 378, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 381, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 381, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 387, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 387, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 390, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 390, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 396, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 396, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 402, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 402, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 405, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 405, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 411, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 411, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 429, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 429, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 432, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 432, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 435, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 435, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 438, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 438, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 441, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 441, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 447, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 447, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 450, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 450, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 453, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 453, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 459, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 459, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 462, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 462, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 1 */
distr[0] = unur_distr_normal(NULL,0);



/* free distributions */
	unur_distr_free(distr[0]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


double pdf(double x, UNUR_DISTR *distr) { return 1.; }
double dpdf(double x, UNUR_DISTR *distr) { return 1.; }
double cdf(double x, UNUR_DISTR *distr) { return 1.; }

double pmf(int k, UNUR_DISTR *distr) { return 1.; }
double icdf(int k, UNUR_DISTR *distr) { return 1.; }

UNUR_DISTR *get_distr_with_pv( void )
{
#define PV_SIZE 200
  double pv[PV_SIZE];
  UNUR_DISTR *distr;
  int i;

  for (i=0; i<PV_SIZE; i++)
    pv[i] = prng_get_next(urng);

  distr = unur_distr_discr_new();
  unur_distr_discr_set_pv(distr,pv,PV_SIZE);

  return distr;

#undef PV_SIZE
} /* end of get_distr_with_pv() */



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_distr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_distr_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_DISTR */
/*---------------------------------------------------------------------------*/
