/*
	file automatically generated by make_test_files.pl
	Fri Jun 14 15:21:43 2002
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DSTD
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_DSTD
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_dstd_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

static int COMPARE_SAMPLE_SIZE = 500;
static int VIOLATE_SAMPLE_SIZE = 20;





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_dstd_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_dstd_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(2997)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("dstd: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 31, (unur_dstd_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 31, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 37, (unur_dstd_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 37, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* not a standard distribution */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 43, (unur_dstd_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 43, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 53, (unur_dstd_set_variant(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 53, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dgt_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 61, (unur_dstd_set_variant(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 61, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dstd_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 71, (unur_dstd_set_variant(par,9999)) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_PAR_VARIANT );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dgt_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 88,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 91, (unur_dstd_chg_pmfparams(gen, fpar, 1)) );
n_tests_failed += check_errorcode( TESTLOG, 91, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dstd_new(distr);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 98,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 101, (unur_dstd_chg_pmfparams(gen,NULL,1)) );
n_tests_failed += check_errorcode( TESTLOG, 101, UNUR_ERR_NULL );

unur_errno = 0;
unur_dstd_chg_pmfparams(gen,fpar,UNUR_DISTR_MAXPARAMS+10);
n_tests_failed += check_errorcode( TESTLOG, 104, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 107, (unur_dstd_chg_pmfparams(gen,fpar,-1)) );
n_tests_failed += check_errorcode( TESTLOG, 107, UNUR_ERR_DISTR_NPARAMS );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 118, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 118, UNUR_ERR_NULL );
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[24];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 24 */
fpm[0] = 50.;
fpm[1] = 0.4;
distr[1] = unur_distr_binomial(fpm,2);


fpm[0] = 100.;
fpm[1] = 0.87;
distr[2] = unur_distr_binomial(fpm,2);


fpm[0] = 100.;
fpm[1] = 0.01;
distr[3] = unur_distr_binomial(fpm,2);


fpm[0] = 10000.;
fpm[1] = 0.7 ;
distr[4] = unur_distr_binomial(fpm,2);


fpm[0] = 0.5;
distr[0] = unur_distr_geometric(fpm,1);


fpm[0] = 0.001;
distr[11] = unur_distr_geometric(fpm,1);


fpm[0] = 0.999;
distr[12] = unur_distr_geometric(fpm,1);


fpm[0] = 10000.;
fpm[1] = 100. ;
fpm[2] = 50. ;
distr[5] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 10000.;
fpm[1] = 9000. ;
fpm[2] = 50. ;
distr[6] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 10000.;
fpm[1] = 80. ;
fpm[2] = 90. ;
distr[7] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 10000.;
fpm[1] = 1000. ;
fpm[2] = 150. ;
distr[8] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 10000.;
fpm[1] = 9000. ;
fpm[2] = 500. ;
distr[9] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 10000.;
fpm[1] = 9000. ;
fpm[2] = 250. ;
distr[10] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 0.001;
distr[13] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.1;
distr[14] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.5;
distr[15] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.9;
distr[16] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.999;
distr[17] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.001;
distr[18] = unur_distr_poisson(fpm,1);


fpm[0] = 0.1;
distr[19] = unur_distr_poisson(fpm,1);


fpm[0] = 0.5;
distr[20] = unur_distr_poisson(fpm,1);


fpm[0] = 1.;
distr[21] = unur_distr_poisson(fpm,1);


fpm[0] = 10.;
distr[22] = unur_distr_poisson(fpm,1);


fpm[0] = 1000.;
distr[23] = unur_distr_poisson(fpm,1);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 144 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [1] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[1]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[1]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [2] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[2]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[2]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [3] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[3]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[3]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[4]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[4]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[0]);
if (!unur_dstd_set_variant(par,UNUR_STDGEN_INVERSION)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[0]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[11]);
if (!unur_dstd_set_variant(par,UNUR_STDGEN_INVERSION)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[11]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[12]);
if (!unur_dstd_set_variant(par,UNUR_STDGEN_INVERSION)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[12]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [5] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[5]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[5]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[6]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[6]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[7]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[7]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[8]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[8]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[9]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[9]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [10] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[10]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[10]);
fpm[0] = 200;
fpm[1] = 50;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[13]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[13]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [14] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[14]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[14]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [14] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[14]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[14]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [16] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[16]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[16]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [17] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[17]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[17]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [18] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[18]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[18]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[18]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[18]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [19] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[19]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[19]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[19]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[19]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [20] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[20]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[20]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[20]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[20]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[21]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[21]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[21]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[21]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [22] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[22]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[22]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[22]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[22]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[23]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[23]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[23]);
fpm[0] = 0.2;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dstd_new(distr[23]);
fpm[0] = 20;
fpm[1] = 0.5;
	gen = unur_init(par);
	if (gen) {
unur_dstd_chg_pmfparams(gen,fpm,2);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("\n(timing) "); fflush(stdout);

/* timing tests: 144 */

	unur_set_default_debug(0u);
{
	double time_setup, time_sample;
	double timing_result_setup[6];
	double timing_result_marginal[6];
	int i;

	fprintf( TESTLOG,"\nTimings (setup / marginal generation times in micro seconds):\n");
	for (i=0; i<6; i++)
		fprintf( TESTLOG, "  [    %1d    ]  ", i);
	fprintf( TESTLOG,"\n");
/* distribution [18] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[18]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[18]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[18], timing_result_setup, timing_result_marginal, 6 );

/* distribution [19] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[19]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[19]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[19], timing_result_setup, timing_result_marginal, 6 );

/* distribution [20] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[20]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[20]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[20], timing_result_setup, timing_result_marginal, 6 );

/* distribution [21] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[21]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[21]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[21], timing_result_setup, timing_result_marginal, 6 );

/* distribution [22] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[22]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[22]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[22], timing_result_setup, timing_result_marginal, 6 );

/* distribution [23] */

	for (i=0; i<6; i++) timing_result_setup[i] = -1.;
	for (i=0; i<6; i++) timing_result_marginal[i] = -1.;
	unur_errno = 0;
par = unur_dstd_new(distr[23]);
if (!unur_dstd_set_variant(par,1)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[1] = time_setup;
	if (gen) timing_result_marginal[1] = time_sample;
	unur_free(gen);

	unur_errno = 0;
par = unur_dstd_new(distr[23]);
if (!unur_dstd_set_variant(par,2)) { free(par); par = NULL; }
	gen = unur_test_timing(par,5,&time_setup,&time_sample,0,TESTLOG);
	if (gen) timing_result_setup[2] = time_setup;
	if (gen) timing_result_marginal[2] = time_sample;
	unur_free(gen);

	print_timing_results( TESTLOG, 0, distr[23], timing_result_setup, timing_result_marginal, 6 );

}

/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_dstd_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_dstd_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_DSTD */
/*---------------------------------------------------------------------------*/
