/*
	file automatically generated by make_test_files.pl
	Fri Jun 14 15:21:45 2002
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for TABL
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_TABL
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_tabl_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */
double pdf( double x, UNUR_DISTR *distr );

double pdf_bimodal( double x, UNUR_DISTR *distr );
double dpdf_bimodal( double x, UNUR_DISTR *distr );

double pdf_negative( double x, UNUR_DISTR *distr );
double dpdf_negative( double x, UNUR_DISTR *distr );

double pdf_partnegative( double x, UNUR_DISTR *distr );
double dpdf_partnegative( double x, UNUR_DISTR *distr );

int unur_tabl_set_pedantic( struct unur_par *par, int pedantic );

static int COMPARE_SAMPLE_SIZE = 10000;
static int VIOLATE_SAMPLE_SIZE = 20;





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_tabl_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_tabl_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(1007)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("tabl: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 43, (unur_tabl_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 43, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 49, (unur_tabl_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 49, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
/* pdf */
n_tests_failed += check_expected_NULL( TESTLOG, 56, (unur_tabl_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 56, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   double slopes[2] = {0.,1.};
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 67, (unur_tabl_set_variant_setup( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 67, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 70, (unur_tabl_set_variant_splitmode( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 70, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 73, (unur_tabl_set_areafraction( par, 0.2 )) );
n_tests_failed += check_errorcode( TESTLOG, 73, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 76, (unur_tabl_set_nstp( par, 30 )) );
n_tests_failed += check_errorcode( TESTLOG, 76, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 79, (unur_tabl_set_slopes( par, slopes, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 79, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 82, (unur_tabl_set_boundary( par, 0.,1. )) );
n_tests_failed += check_errorcode( TESTLOG, 82, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 85, (unur_tabl_set_guidefactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 85, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 88, (unur_tabl_set_max_sqhratio( par, 0.95 )) );
n_tests_failed += check_errorcode( TESTLOG, 88, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 91, (unur_tabl_set_max_intervals( par, 100 )) );
n_tests_failed += check_errorcode( TESTLOG, 91, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 94, (unur_tabl_set_verify( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 94, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   double slopes[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 104, (unur_tabl_set_variant_setup( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 104, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 107, (unur_tabl_set_variant_splitmode( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 107, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 110, (unur_tabl_set_areafraction( par, 0.2 )) );
n_tests_failed += check_errorcode( TESTLOG, 110, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 113, (unur_tabl_set_nstp( par, 30 )) );
n_tests_failed += check_errorcode( TESTLOG, 113, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 116, (unur_tabl_set_slopes( par, slopes, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 116, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 119, (unur_tabl_set_boundary( par, 0.,1. )) );
n_tests_failed += check_errorcode( TESTLOG, 119, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 122, (unur_tabl_set_guidefactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 122, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 125, (unur_tabl_set_max_sqhratio( par, 0.95 )) );
n_tests_failed += check_errorcode( TESTLOG, 125, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 128, (unur_tabl_set_max_intervals( par, 100 )) );
n_tests_failed += check_errorcode( TESTLOG, 128, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 131, (unur_tabl_set_verify( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 131, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double slopesA[4] = {0.,1.,0.5,2.};
   double slopesB[4] = {1.,2.,0.,1.};
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_tabl_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 143, (unur_tabl_set_variant_setup( par, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 143, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 146, (unur_tabl_set_variant_setup( par, 3 )) );
n_tests_failed += check_errorcode( TESTLOG, 146, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 149, (unur_tabl_set_variant_splitmode( par, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 149, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 152, (unur_tabl_set_variant_splitmode( par, 4 )) );
n_tests_failed += check_errorcode( TESTLOG, 152, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 155, (unur_tabl_set_areafraction( par, -0.1 )) );
n_tests_failed += check_errorcode( TESTLOG, 155, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 158, (unur_tabl_set_nstp( par, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 158, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 161, (unur_tabl_set_slopes( par, slopesA, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 161, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 164, (unur_tabl_set_slopes( par, slopesA, 4 )) );
n_tests_failed += check_errorcode( TESTLOG, 164, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 167, (unur_tabl_set_slopes( par, slopesB, 4 )) );
n_tests_failed += check_errorcode( TESTLOG, 167, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 170, (unur_tabl_set_boundary( par, 2., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 170, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 173, (unur_tabl_set_boundary( par, 0., UNUR_INFINITY )) );
n_tests_failed += check_errorcode( TESTLOG, 173, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 176, (unur_tabl_set_boundary( par, -UNUR_INFINITY, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 176, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 179, (unur_tabl_set_guidefactor( par, -0.1 )) );
n_tests_failed += check_errorcode( TESTLOG, 179, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 182, (unur_tabl_set_max_sqhratio( par, -0.1 )) );
n_tests_failed += check_errorcode( TESTLOG, 182, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 185, (unur_tabl_set_max_sqhratio( par, 1.1 )) );
n_tests_failed += check_errorcode( TESTLOG, 185, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 188, (unur_tabl_set_max_intervals( par, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 188, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 199,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 202, (unur_tabl_get_sqhratio(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 202, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 205, (unur_tabl_get_hatarea(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 205, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 208, (unur_tabl_get_squeezearea(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 208, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 220,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 223, (unur_tabl_chg_verify(gen,1)) );
n_tests_failed += check_errorcode( TESTLOG, 223, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 234, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 234, UNUR_ERR_NULL );
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_normal(NULL,0); 
   par = NULL; 

unur_errno = 0;
par = unur_tabl_new(distr);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 319, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
par = unur_tabl_new(distr);
unur_tabl_set_verify(par,1);
n_tests_failed += compare_double_sequence_par( TESTLOG, 324, urng, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[27];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 27 */
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);


fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);


fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);


distr[7] = unur_distr_cauchy(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);


distr[23] = unur_distr_exponential(NULL,0);


fpm[0] = 30.;
fpm[1] = -5.;
distr[24] = unur_distr_exponential(fpm,2);


fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);


fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);


fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);


fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);


fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);


fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);


distr[25] = unur_distr_laplace(NULL,0);


fpm[0] = -10.;
fpm[1] = 100.;
distr[26] = unur_distr_laplace(fpm,2);


distr[17] = unur_distr_normal(NULL,0);


fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);


fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);


distr[20] = unur_distr_uniform(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);


distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 54 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[1]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[2]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[3]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[4]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[5]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[6]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[7]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[7]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[8]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[23]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[23]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[24]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[24]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[9]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[10]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[11]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[12]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[13]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[14]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[15]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[16]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[25]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[25]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[26]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[26]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[17]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[17]);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [18] */

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[19]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[20]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[21]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[22]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 54 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[0]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[1]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[2]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[3]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[4]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[5]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[6]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[7]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[7]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[8]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[8]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[23]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[23]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[24]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[24]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[9]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[10]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[11]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[12]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[13]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[14]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[15]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[16]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[25]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[25]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[26]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[26]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[17]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{double slopes[4] = {0.,-0.5,0.,2.};
par = unur_tabl_new(distr[17]);
	unur_tabl_set_pedantic(par,0);
unur_tabl_set_slopes(par,slopes,2);}
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [18] */

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[19]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[20]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[21]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tabl_new(distr[22]);
	unur_tabl_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_tabl_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/* pdf of bimodal density */
double pdf_bimodal( double x, UNUR_DISTR *distr )
{ 
	return ( exp(-(x-1.)*(x-1.)) + exp(-(x+1.)*(x+1.)) );
}
double dpdf_bimodal( double x, UNUR_DISTR *distr )
{ 
	return ( -2. * (x-1.) * exp(-(x-1.)*(x-1.)) -2. * (x+1.) * exp(-(x+1.)*(x+1.)) );
}

/* pdf with negative value */
double pdf_negative( double x, UNUR_DISTR *distr )  { return (-x*x); }
double dpdf_negative( double x, UNUR_DISTR *distr ) { return (-2.*x); }

/* pdf with partial negative value */
double pdf_partnegative( double x, UNUR_DISTR *distr )  
{ 
	return ((x>-0.89 && x<0.89) ? -1.: exp(-x*x)); 
}
double dpdf_partnegative( double x, UNUR_DISTR *distr ) 
{
	return ((x>-0.89 && x<0.89) ?0.: -2.*x*exp(-x*x));
}

/* pdf of density */
double pdf( double x, UNUR_DISTR *distr )
{ 
	return exp(-x*x/2.);
} /* end of pdf */

/* no such function */
int unur_tabl_set_pedantic( struct unur_par *par, int pedantic )
{ return 1; }





/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_tabl_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_TABL */
/*---------------------------------------------------------------------------*/
